@@ -83,6 +83,7 @@ static gchar   *zenity_list_print_column;
 static gchar   *zenity_list_hide_column;
 static gboolean zenity_list_hide_header;
 static gboolean zenity_list_imagelist;
+static gboolean zenity_list_mid_search;
 
 #ifdef HAVE_LIBNOTIFY
 /* Notification Dialog Options */
@@ -98,6 +99,7 @@ static gboolean zenity_progress_pulsate;
 static gboolean zenity_progress_auto_close;
 static gboolean zenity_progress_auto_kill;
 static gboolean zenity_progress_no_cancel;
+static gboolean zenity_progress_time_remaining;
 
 /* Question Dialog Options */
 static gboolean zenity_question_active;
@@ -111,6 +113,7 @@ static gboolean zenity_text_auto_scroll;
 
 #ifdef HAVE_WEBKITGTK
 static gboolean zenity_text_enable_html;
+static gboolean zenity_text_no_interaction;
 static gchar   *zenity_text_url;
 #endif
 
@@ -650,6 +653,15 @@ static GOptionEntry list_options[] = {
     N_("Hides the column headers"),
     NULL
   },
+  {
+    "mid-search",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_list_mid_search,
+    N_("Change list default search function searching for text in the middle, not on the beginning"),
+    NULL
+  },
   { 
     NULL 
   } 
@@ -767,6 +779,15 @@ static GOptionEntry progress_options[] = {
    N_("Hide Cancel button"),
    NULL
   },
+  {
+    "time-remaining",
+    '\0',
+    0,
+    G_OPTION_ARG_NONE,
+    &zenity_progress_time_remaining,
+    N_("Estimate when progress will reach 100%"),
+    NULL
+  },
   { 
     NULL 
   }
@@ -895,6 +916,15 @@ static GOptionEntry text_options[] = {
     N_("Enable html support"),
     NULL
   },
+  {
+    "no-interaction",
+    '\0',
+    G_OPTION_FLAG_NOALIAS,
+    G_OPTION_ARG_NONE,
+    &zenity_text_no_interaction,
+    N_("Do not enable user interaction with the WebView. Only works if you use --html option"),
+    NULL
+  },
   {
     "url",
     '\0',
@@ -1521,6 +1551,7 @@ zenity_list_pre_callback (GOptionContext *context,
   zenity_list_hide_header = FALSE;
   zenity_list_print_column = NULL;
   zenity_list_hide_column = NULL;
+  zenity_list_mid_search = FALSE;
 
   return TRUE;
 }
@@ -1551,6 +1582,7 @@ zenity_progress_pre_callback (GOptionContext *context,
   zenity_progress_auto_close = FALSE;
   zenity_progress_auto_kill = FALSE;
   zenity_progress_no_cancel = FALSE;
+  zenity_progress_time_remaining = FALSE;
   return TRUE;
 }
 
@@ -1578,6 +1610,7 @@ zenity_text_pre_callback (GOptionContext *context,
   zenity_text_auto_scroll = FALSE;
 #ifdef HAVE_WEBKITGTK
   zenity_text_enable_html = FALSE;
+  zenity_text_no_interaction = FALSE;
   zenity_text_url = NULL;
 #endif
   return TRUE;
@@ -1865,6 +1898,7 @@ zenity_list_post_callback (GOptionContext *context,
     results->tree_data->hide_column = zenity_list_hide_column;
     results->tree_data->hide_header = zenity_list_hide_header;
     results->tree_data->separator = zenity_general_separator;
+    results->tree_data->mid_search = zenity_list_mid_search;
   } else {
     if (zenity_list_columns)
       zenity_option_error (zenity_option_get_name (list_options, &zenity_list_columns),
@@ -1893,6 +1927,9 @@ zenity_list_post_callback (GOptionContext *context,
     if (zenity_list_hide_header)
       zenity_option_error (zenity_option_get_name (list_options, &zenity_list_hide_header),
                            ERROR_SUPPORT);
+    if (zenity_list_mid_search)
+      zenity_option_error (zenity_option_get_name (list_options, &zenity_list_mid_search),
+                           ERROR_SUPPORT);
   }
     
   return TRUE;
@@ -1935,6 +1972,7 @@ zenity_progress_post_callback (GOptionContext *context,
     results->progress_data->autokill = zenity_progress_auto_kill;
     results->progress_data->percentage = zenity_progress_percentage;
     results->progress_data->no_cancel = zenity_progress_no_cancel;
+    results->progress_data->time_remaining = zenity_progress_time_remaining;
   } else {
     if (zenity_progress_pulsate)
       zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_pulsate),
@@ -1951,9 +1989,14 @@ zenity_progress_post_callback (GOptionContext *context,
     if (zenity_progress_auto_kill)
       zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_auto_kill),
                            ERROR_SUPPORT);
+
     if (zenity_progress_no_cancel)
       zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_no_cancel),
 			   ERROR_SUPPORT);
+
+    if (zenity_progress_time_remaining)
+      zenity_option_error (zenity_option_get_name (progress_options, &zenity_progress_time_remaining),
+                           ERROR_SUPPORT);
   }
     
   return TRUE;
@@ -1996,6 +2039,7 @@ zenity_text_post_callback (GOptionContext *context,
     results->text_data->auto_scroll = zenity_text_auto_scroll;
 #ifdef HAVE_WEBKITGTK
     results->text_data->html = zenity_text_enable_html;
+    results->text_data->no_interaction = zenity_text_no_interaction;
     results->text_data->url = zenity_text_url;
 #endif
   } else {
