@@ -152,9 +152,9 @@ void zenity_password_dialog (ZenityData *data, ZenityPasswordData *password_data
   zenity_util_show_dialog (dialog, data->attach);
 
   if (data->timeout_delay > 0) {
-    g_timeout_add (data->timeout_delay * 1000,
-                   (GSourceFunc) zenity_util_timeout_handle,
-                   dialog);
+    g_timeout_add_seconds (data->timeout_delay,
+                           (GSourceFunc) zenity_util_timeout_handle,
+                           dialog);
   }
   gtk_main();
 }
