@@ -29,6 +29,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <signal.h>
+#include <time.h>
 #include "zenity.h"
 #include "util.h"
 
@@ -67,8 +68,41 @@ zenity_progress_pulsate_start (GObject *progress_bar)
 {
   if (pulsate_timeout == -1) {
     pulsate_timeout = g_timeout_add (100,
-                                     zenity_progress_pulsate_progress_bar,
-                                     progress_bar);
+                                             zenity_progress_pulsate_progress_bar,
+                                             progress_bar);
+  }
+}
+
+static void
+zenity_progress_update_time_remaining (ZenityProgressData *progress_data)
+{
+  static GObject *progress_time = NULL;
+  static time_t start_time = (time_t)(-1);
+  float percentage = progress_data->percentage;
+
+  if (progress_time == NULL)
+    progress_time = gtk_builder_get_object (builder, "zenity_progress_time");
+  if (start_time == (time_t)(-1) || percentage <= 0.0 || percentage >= 100.0) {
+    start_time = time(NULL);
+    gtk_label_set_text (GTK_LABEL (progress_time), "");
+  } else {
+    time_t current_time = time (NULL);
+    time_t elapsed_time = current_time - start_time;
+    time_t total_time = (time_t) (100.0*elapsed_time/progress_data->percentage);
+    time_t remaining_time = total_time - elapsed_time;
+    gulong hours, minutes, seconds;
+    gchar *remaining_message;
+
+    seconds = (gulong) (remaining_time%60);
+    remaining_time /= 60;
+    minutes = (gulong) (remaining_time%60);
+    remaining_time /= 60;
+    hours = (gulong) remaining_time;
+
+    remaining_message = g_strdup_printf (_("Time remaining: %lu:%02lu:%02lu"),
+					 hours, minutes, seconds);
+    gtk_label_set_text (GTK_LABEL (progress_time), remaining_message);
+    g_free (remaining_message);
   }
 }
 
@@ -160,6 +194,9 @@ zenity_progress_handle_stdin (GIOChannel   *channel,
 
         progress_data->percentage = percentage;
 
+	if (progress_data->time_remaining == TRUE)
+	  zenity_progress_update_time_remaining (progress_data);
+
         if (percentage == 100) {
           GObject *button;
 
@@ -225,6 +262,12 @@ zenity_progress_read_info (ZenityProgressData *progress_data)
   }
 }
 
+static void
+zenity_text_size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpointer data)
+{
+  gtk_widget_set_size_request (widget, allocation->width/2, -1);
+}
+
 void
 zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 {
@@ -244,6 +287,8 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 
   gtk_builder_connect_signals (builder, NULL);
 
+  text = gtk_builder_get_object (builder, "zenity_progress_text");
+
   dialog = GTK_WIDGET (gtk_builder_get_object (builder,
                                                "zenity_progress_dialog"));
 
@@ -258,6 +303,16 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
   if (data->width > -1 || data->height > -1)
     gtk_window_set_default_size (GTK_WINDOW (dialog), data->width, data->height);
 
+  if (data->width > -1) {
+    gtk_widget_set_size_request (GTK_WIDGET (text), data->width, -1);
+  }
+  else {
+    g_signal_connect_after (G_OBJECT (text), "size-allocate",
+                            G_CALLBACK (zenity_text_size_allocate), data);
+    g_signal_connect_after (G_OBJECT (progress_bar), "size-allocate",
+                            G_CALLBACK (zenity_text_size_allocate), data);
+  }
+
   if (data->modal)
     gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
@@ -275,8 +330,6 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
                           gtk_image_new_from_stock (GTK_STOCK_CANCEL, GTK_ICON_SIZE_BUTTON));
   }
 
-  text = gtk_builder_get_object (builder, "zenity_progress_text");
-
   if (progress_data->dialog_text)
     gtk_label_set_markup (GTK_LABEL (text), g_strcompress (progress_data->dialog_text));
 
