@@ -23,6 +23,7 @@
 
 #include "config.h"
 
+#include <gio/gio.h>
 #include "zenity.h"
 #include "util.h"
 
@@ -59,7 +60,7 @@ zenity_configure_webkit (WebKitWebView *web_view)
   g_object_set(G_OBJECT(settings), "enable-developer-extras",     FALSE, NULL);
   /* unexisting property? g_object_set(G_OBJECT(settings), "enable-dns-prefetching",      FALSE, NULL);*/
   g_object_set(G_OBJECT(settings), "enable-dom-paste",            FALSE, NULL);
-  g_object_set(G_OBJECT(settings), "enable-file-access-from-file-uris", FALSE, NULL);
+  g_object_set(G_OBJECT(settings), "enable-file-access-from-file-uris", TRUE, NULL);
   /* unexisting property? g_object_set(G_OBJECT(settings), "enable-frame-flattening",     FALSE, NULL);*/
   /* unexisting property? g_object_set(G_OBJECT(settings), "enable-fullscreen",           FALSE, NULL);*/
   g_object_set(G_OBJECT(settings), "enable-html5-database",       FALSE, NULL);
@@ -114,6 +115,11 @@ zenity_text_webview_decision_request (WebKitWebView             *webkitwebview,
                                       gpointer                   user_data)
 {
   webkit_web_policy_decision_ignore (policy_decision);
+  if (!zen_text_data->no_interaction &&
+      webkit_web_navigation_action_get_reason (navigation_action) == WEBKIT_WEB_NAVIGATION_REASON_LINK_CLICKED) {
+    g_app_info_launch_default_for_uri (webkit_web_navigation_action_get_original_uri(navigation_action),
+                                       NULL, NULL);
+  }
   return TRUE;
 }
 
@@ -134,11 +140,13 @@ zenity_text_handle_stdin (GIOChannel  *channel,
                           gpointer     data)
 {
   static GtkTextBuffer *buffer;
+  static GtkTextView *text_view;
   gchar buf[1024];
 
   gsize len;
 
-  buffer = GTK_TEXT_BUFFER (data);
+  text_view = GTK_TEXT_VIEW (data);
+  buffer = gtk_text_view_get_buffer (text_view);
 
   if ((condition & G_IO_IN) || (condition & (G_IO_IN | G_IO_HUP))) {
     GError *error = NULL;
@@ -179,6 +187,12 @@ zenity_text_handle_stdin (GIOChannel  *channel,
       } else {
         gtk_text_buffer_insert (buffer, &end, buf, len);
       }
+      if (zen_text_data->auto_scroll) {
+        GtkTextMark *mark = NULL;
+        mark = gtk_text_buffer_get_insert (buffer);
+        if (mark != NULL)
+          gtk_text_view_scroll_to_mark (text_view, mark, 0.0, FALSE, 0, 0);
+      }
     }
   }
 
@@ -186,14 +200,14 @@ zenity_text_handle_stdin (GIOChannel  *channel,
 }
 
 static void
-zenity_text_fill_entries_from_stdin (GtkTextBuffer *text_buffer)
+zenity_text_fill_entries_from_stdin (GtkTextView *text_view)
 {
   GIOChannel *channel; 
 
   channel = g_io_channel_unix_new (0);
-  g_io_channel_set_encoding (channel, NULL, NULL);
+  g_io_channel_set_encoding (channel, "UTF-8", NULL);
   g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, NULL);
-  g_io_add_watch (channel, G_IO_IN | G_IO_HUP, zenity_text_handle_stdin, text_buffer);
+  g_io_add_watch (channel, G_IO_IN | G_IO_HUP, zenity_text_handle_stdin, text_view);
 }
 
 void
@@ -233,7 +247,7 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (zenity_text_dialog_response), data);
-	
+
   if (data->dialog_title)
     gtk_window_set_title (GTK_WINDOW (dialog), data->dialog_title);
 
@@ -257,7 +271,7 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
   if (text_data->uri)
     zenity_util_fill_file_buffer (text_buffer, text_data->uri);
   else
-    zenity_text_fill_entries_from_stdin (GTK_TEXT_BUFFER (text_buffer));
+    zenity_text_fill_entries_from_stdin (GTK_TEXT_VIEW(text_view));
 
   if (text_data->editable)
     zen_text_data->buffer = text_buffer;
@@ -308,15 +322,24 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
     }
     else
     {
+      gchar *cwd;
+      gchar *dirname;
+      gchar *dirname_uri;
+      dirname = text_data->uri ? g_path_get_dirname (text_data->uri) : g_strdup ("/");
+      cwd = g_get_current_dir ();
+      dirname_uri = g_strconcat ("file://", cwd, "/", dirname, "/", NULL);
+      g_free (cwd);
+      g_free (dirname);
       gtk_text_buffer_get_start_iter (text_buffer, &start_iter);
       gtk_text_buffer_get_end_iter (text_buffer, &end_iter);
       content = gtk_text_buffer_get_text (text_buffer, &start_iter, &end_iter, TRUE);
-      webkit_web_view_load_string (WEBKIT_WEB_VIEW(web_kit), content, "text/html", "UTF-8", NULL);
+      webkit_web_view_load_string (WEBKIT_WEB_VIEW(web_kit), content, "text/html", "UTF-8", dirname_uri);
+      g_free (dirname_uri);
       g_free (content);
     }
 
     // We don't want user to click on links and navigate to another page.
-    // So, when page finish load, we block requests.
+    // So, when the page finishes loading, we take handle of the requests.
 
     g_signal_connect (G_OBJECT (web_kit), "document-load-finished",
                       G_CALLBACK (zenity_text_webview_load_finished), NULL); 
@@ -326,7 +349,8 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
     gtk_widget_show (GTK_WIDGET (web_kit));
   }
 #endif
-  zenity_util_show_dialog (dialog);
+
+  zenity_util_show_dialog (dialog, data->attach);
 
   g_object_unref (builder);
 
