@@ -139,6 +139,7 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
   static gboolean editable;
   static gboolean toggles;
   static gboolean first_time = TRUE;
+  GIOStatus status = G_IO_STATUS_NORMAL;
 
   tree_view = GTK_TREE_VIEW (data);
   n_columns = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tree_view), "n_columns"));
@@ -152,7 +153,7 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
     gtk_list_store_append (GTK_LIST_STORE (model), &iter);
   }
 
-  if ((condition == G_IO_IN) || (condition == G_IO_IN + G_IO_HUP)) {
+  if ((condition & G_IO_IN) == G_IO_IN) {
     GString *string;
     GError *error = NULL;
 
@@ -161,8 +162,6 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
     while ((g_io_channel_get_flags(channel) & G_IO_FLAG_IS_READABLE) != G_IO_FLAG_IS_READABLE)
       ;
     do {
-      gint status;
-
       do {
         if (g_io_channel_get_flags(channel) & G_IO_FLAG_IS_READABLE)
           status = g_io_channel_read_line_string (channel, string, NULL, &error);
@@ -221,11 +220,11 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
 
       column_count++;
 
-    } while (g_io_channel_get_buffer_condition (channel) == G_IO_IN); 
+    } while ((g_io_channel_get_buffer_condition (channel) & G_IO_IN) == G_IO_IN && status != G_IO_STATUS_EOF); 
     g_string_free (string, TRUE);
   }
 
-  if ((condition != G_IO_IN) && (condition != G_IO_IN + G_IO_HUP)) {
+  if ((condition & G_IO_IN) != G_IO_IN || status == G_IO_STATUS_EOF) {
     g_io_channel_shutdown (channel, TRUE, NULL);
     return FALSE;
   }
@@ -299,6 +298,17 @@ zenity_tree_fill_entries (GtkTreeView  *tree_view,
   }
 }
 
+static gboolean
+zenity_mid_search_func (GtkTreeModel *model, gint column,
+                    const gchar *key, GtkTreeIter *iter,
+                    gpointer search_data)
+{
+    gchar *iter_string = NULL;
+    gtk_tree_model_get (model, iter, column, &iter_string, -1); 
+    return  ! g_strrstr (g_utf8_strdown(iter_string, -1), 
+                         g_utf8_strdown(key, -1)) != NULL;
+}
+
 static void
 zenity_cell_edited_callback (GtkCellRendererText *cell, 
                              const gchar         *path_string, 
@@ -563,7 +573,10 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
       zenity_tree_fill_entries_from_stdin (GTK_TREE_VIEW (tree_view), n_columns, FALSE, tree_data->editable);
   }
 
-  zenity_util_show_dialog (dialog);
+  zenity_util_show_dialog (dialog, data->attach);
+
+  if (tree_data->mid_search)
+    gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(tree_view), (GtkTreeViewSearchEqualFunc) zenity_mid_search_func, model, NULL);
 
   if(data->timeout_delay > 0) {
     g_timeout_add_seconds (data->timeout_delay, (GSourceFunc) zenity_util_timeout_handle, dialog);
@@ -710,12 +723,6 @@ zenity_tree_row_activated (GtkTreeView *tree_view, GtkTreePath *tree_path,
                            GtkTreeViewColumn *tree_col, gpointer data)
 {
   ZenityData *zen_data = data;
-  GtkTreeSelection *selection; 
-  
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
-  gtk_tree_selection_selected_foreach (selection, 
-                                       (GtkTreeSelectionForeachFunc) zenity_tree_dialog_get_selected, 
-                                       GTK_TREE_VIEW (tree_view));
  
   zenity_tree_dialog_output ();
   zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
