@@ -105,6 +105,7 @@ typedef struct {
   gboolean autokill;
   gdouble  percentage;
   gboolean no_cancel;
+  gboolean time_remaining;
 } ZenityProgressData;
 
 typedef struct {
@@ -117,6 +118,7 @@ typedef struct {
   gchar         *checkbox;
 #ifdef HAVE_WEBKITGTK
   gboolean       html;
+  gboolean       no_interaction;
   gchar         *url;
 #endif
 } ZenityTextData;
@@ -131,6 +133,7 @@ typedef struct {
   gchar        *separator;
   gboolean      multi;
   gboolean      editable;
+  gboolean      mid_search;
   gchar	       *print_column;
   gchar	       *hide_column;
   const gchar **data;
